package org.apache.maven.plugins.war.stub;

/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

import java.util.ArrayList;
import java.util.List;

import org.apache.maven.model.Resource;

public class ResourceStub
    extends Resource
{
    /**
     * 
     */
    private static final long serialVersionUID = 7685068931840967662L;

    private String directory;

    public List<String> getIncludes()
    {
        return new ArrayList<>();
    }

    public List<String> getExcludes()
    {
        return new ArrayList<>();
    }

    public void setDirectory( String _directory )
    {
        directory = _directory;
    }

    public String getDirectory()
    {
        return directory;
    }

}
